=== modified file 'breezy/branch.py'
--- breezy/branch.py	2018-03-25 00:39:16 +0000
+++ breezy/branch.py	2018-03-31 19:16:26 +0000
@@ -1201,7 +1201,7 @@
         return result
 
     def sprout(self, to_controldir, revision_id=None, repository_policy=None,
-            repository=None):
+            repository=None, lossy=False):
         """Create a new line of development from the branch, into to_controldir.
 
         to_controldir controls the branch format.
@@ -1213,6 +1213,8 @@
             repository_policy.requires_stacking()):
             to_controldir._format.require_stacking(_skip_repo=True)
         result = to_controldir.create_branch(repository=repository)
+        if lossy:
+            raise errors.LossyPushToSameVCS(self, result)
         with self.lock_read(), result.lock_write():
             if repository_policy is not None:
                 repository_policy.configure_branch(result)

=== modified file 'breezy/builtins.py'
--- breezy/builtins.py	2018-04-03 02:45:54 +0000
+++ breezy/builtins.py	2018-04-06 19:28:56 +0000
@@ -1401,6 +1401,8 @@
                " that support it."),
         Option('overwrite-tags',
               help="Overwrite tags only."),
+        Option('lossy', help="Allow lossy push, i.e. dropping metadata "
+                             "that can't be represented in the target.")
         ]
     takes_args = ['location?']
     encoding_type = 'replace'
@@ -1409,7 +1411,7 @@
         create_prefix=False, verbose=False, revision=None,
         use_existing_dir=False, directory=None, stacked_on=None,
         stacked=False, strict=None, no_tree=False,
-        overwrite_tags=False):
+        overwrite_tags=False, lossy=False):
         from .push import _show_push_branch
 
         if overwrite:
@@ -1476,7 +1478,8 @@
         _show_push_branch(br_from, revision_id, location, self.outf,
             verbose=verbose, overwrite=overwrite, remember=remember,
             stacked_on=stacked_on, create_prefix=create_prefix,
-            use_existing_dir=use_existing_dir, no_tree=no_tree)
+            use_existing_dir=use_existing_dir, no_tree=no_tree,
+            lossy=lossy)
 
 
 class cmd_branch(Command):

=== modified file 'breezy/bzr/bzrdir.py'
--- breezy/bzr/bzrdir.py	2018-03-31 23:39:19 +0000
+++ breezy/bzr/bzrdir.py	2018-04-06 19:28:56 +0000
@@ -348,7 +348,8 @@
     def sprout(self, url, revision_id=None, force_new_repo=False,
                recurse='down', possible_transports=None,
                accelerator_tree=None, hardlink=False, stacked=False,
-               source_branch=None, create_tree_if_local=True):
+               source_branch=None, create_tree_if_local=True,
+               lossy=False):
         """Create a copy of this controldir prepared for use as a new line of
         development.
 
@@ -385,7 +386,7 @@
     def _sprout(self, op, url, revision_id=None, force_new_repo=False,
                recurse='down', possible_transports=None,
                accelerator_tree=None, hardlink=False, stacked=False,
-               source_branch=None, create_tree_if_local=True):
+               source_branch=None, create_tree_if_local=True, lossy=False):
         add_cleanup = op.add_cleanup
         fetch_spec_factory = fetch.FetchSpecFactory()
         if revision_id is not None:

=== modified file 'breezy/controldir.py'
--- breezy/controldir.py	2018-03-31 23:39:19 +0000
+++ breezy/controldir.py	2018-04-06 19:28:56 +0000
@@ -372,7 +372,8 @@
     def sprout(self, url, revision_id=None, force_new_repo=False,
                recurse='down', possible_transports=None,
                accelerator_tree=None, hardlink=False, stacked=False,
-               source_branch=None, create_tree_if_local=True):
+               source_branch=None, create_tree_if_local=True,
+               lossy=False):
         """Create a copy of this controldir prepared for use as a new line of
         development.
 
@@ -398,8 +399,8 @@
         """
         raise NotImplementedError(self.sprout)
 
-    def push_branch(self, source, revision_id=None, overwrite=False, 
-        remember=False, create_prefix=False):
+    def push_branch(self, source, revision_id=None, overwrite=False,
+        remember=False, create_prefix=False, lossy=False):
         """Push the source branch into this ControlDir."""
         br_to = None
         # If we can open a branch, use its direct repository, otherwise see
@@ -423,7 +424,7 @@
                 # revision
                 revision_id = source.last_revision()
             repository_to.fetch(source.repository, revision_id=revision_id)
-            br_to = source.sprout(self, revision_id=revision_id)
+            br_to = source.sprout(self, revision_id=revision_id, lossy=lossy)
             if source.get_push_location() is None or remember:
                 # FIXME: Should be done only if we succeed ? -- vila 2012-01-18
                 source.set_push_location(br_to.base)
@@ -443,20 +444,18 @@
                 tree_to = self.open_workingtree()
             except errors.NotLocalUrl:
                 push_result.branch_push_result = source.push(br_to, 
-                    overwrite, stop_revision=revision_id)
+                    overwrite, stop_revision=revision_id, lossy=lossy)
                 push_result.workingtree_updated = False
             except errors.NoWorkingTree:
                 push_result.branch_push_result = source.push(br_to,
-                    overwrite, stop_revision=revision_id)
+                    overwrite, stop_revision=revision_id, lossy=lossy)
                 push_result.workingtree_updated = None # Not applicable
             else:
-                tree_to.lock_write()
-                try:
+                with tree_to.lock_write():
                     push_result.branch_push_result = source.push(
-                        tree_to.branch, overwrite, stop_revision=revision_id)
+                        tree_to.branch, overwrite, stop_revision=revision_id,
+                        lossy=lossy)
                     tree_to.update()
-                finally:
-                    tree_to.unlock()
                 push_result.workingtree_updated = True
             push_result.old_revno = push_result.branch_push_result.old_revno
             push_result.old_revid = push_result.branch_push_result.old_revid

=== modified file 'breezy/push.py'
--- breezy/push.py	2017-07-23 17:17:18 +0000
+++ breezy/push.py	2018-03-31 17:16:06 +0000
@@ -61,7 +61,7 @@
 
 def _show_push_branch(br_from, revision_id, location, to_file, verbose=False,
     overwrite=False, remember=False, stacked_on=None, create_prefix=False,
-    use_existing_dir=False, no_tree=False):
+    use_existing_dir=False, no_tree=False, lossy=False):
     """Push a branch to a location.
 
     :param br_from: the source branch
@@ -79,6 +79,7 @@
         at the destination if they don't already exist
     :param use_existing_dir: if True, proceed even if the destination
         directory exists without a current control directory in it
+    :param lossy: Allow lossy push
     """
     to_transport = transport.get_transport(location)
     try:
@@ -143,8 +144,8 @@
             warning("Ignoring request for a stacked branch as repository "
                     "already exists at the destination location.")
         try:
-            push_result = dir_to.push_branch(br_from, revision_id, overwrite, 
-                remember, create_prefix)
+            push_result = dir_to.push_branch(br_from, revision_id, overwrite,
+                remember, create_prefix, lossy=lossy)
         except errors.DivergedBranches:
             raise errors.BzrCommandError(gettext('These branches have diverged.'
                                     '  See "brz help diverged-branches"'

=== modified file 'breezy/tests/per_controldir/test_push.py'
--- breezy/tests/per_controldir/test_push.py	2017-08-07 21:14:18 +0000
+++ breezy/tests/per_controldir/test_push.py	2018-03-31 19:16:26 +0000
@@ -16,6 +16,8 @@
 
 """Tests for bzrdir implementations - push."""
 
+from ...errors import LossyPushToSameVCS
+
 from breezy.tests.per_controldir import (
     TestCaseWithControlDir,
     )
@@ -39,6 +41,12 @@
         self.assertEqual(dir.open_branch().base,
             tree.branch.get_push_location())
 
+    def test_push_new_branch_lossy(self):
+        tree, rev_1 = self.create_simple_tree()
+        dir = self.make_repository('dir').controldir
+        self.assertRaises(LossyPushToSameVCS, dir.push_branch,
+                          tree.branch, lossy=True)
+
     def test_push_new_empty(self):
         tree = self.make_branch_and_tree('tree')
         dir = self.make_repository('dir').controldir

